@@ -1,5 +1,19 @@
 # Author: Ingo Feinerer
 
+Corpus <-
+function(x, readerControl = list(reader = reader(x), language = "en"))
+{
+    stopifnot(inherits(x, "Source"))
+
+    readerControl <- prepareReader(readerControl, reader(x))
+
+    if ((inherits(x, "DirSource") || inherits(x, "VectorSource")) &&
+        identical(readerControl$reader, readPlain))
+        SimpleCorpus(x, readerControl)
+    else
+        VCorpus(x, readerControl)
+}
+
 PCorpus <-
 function(x,
          readerControl = list(reader = reader(x), language = "en"),
@@ -13,6 +27,7 @@ function(x,
         stop("error in creating database")
     db <- filehash::dbInit(dbControl$dbName, dbControl$dbType)
 
+    x <- open(x)
     tdl <- vector("list", length(x))
     counter <- 1
     while (!eoi(x)) {
@@ -25,15 +40,42 @@ function(x,
         tdl[[counter]] <- meta(doc, "id")
         counter <- counter + 1
     }
+    x <- close(x)
 
-    structure(list(content = tdl,
-                   meta = CorpusMeta(),
-                   dmeta = data.frame(row.names = seq_along(tdl)),
-                   dbcontrol = dbControl),
-              class = c("PCorpus", "Corpus"))
+    p <- list(content = tdl,
+              meta = CorpusMeta(),
+              dmeta = data.frame(row.names = seq_along(tdl)),
+              dbcontrol = dbControl)
+    class(p) <- c("PCorpus", "Corpus")
+    p
+}
+
+SimpleCorpus <-
+function(x, control = list(language = "en"))
+{
+    stopifnot(inherits(x, "Source"))
+
+    if (!is.null(control$reader) && !identical(control$reader, readPlain))
+        warning("custom reader is ignored")
+
+    content <- if (inherits(x, "VectorSource")) {
+        if (is.character(x$content)) x$content else as.character(x$content)
+    } else if (inherits(x, "DirSource")) {
+        setNames(as.character(
+                   lapply(x$filelist,
+                          function(f) paste(readContent(f, x$encoding, "text"),
+                                            collapse = "\n"))
+                   ),
+                 basename(x$filelist))
+    } else
+        stop("unsupported source type")
+    s <- list(content = content,
+              meta = CorpusMeta(language = control$language),
+              dmeta = data.frame(row.names = seq_along(x)))
+    class(s) <- c("SimpleCorpus", "Corpus")
+    s
 }
 
-Corpus <-
 VCorpus <-
 function(x, readerControl = list(reader = reader(x), language = "en"))
 {
@@ -41,11 +83,12 @@ function(x, readerControl = list(reader = reader(x), language = "en"))
 
     readerControl <- prepareReader(readerControl, reader(x))
 
+    x <- open(x)
     tdl <- vector("list", length(x))
     # Check for parallel element access
     if (is.function(getS3method("pGetElem", class(x), TRUE)))
         tdl <- mapply(function(elem, id)
-                          readerControl$reader(elem, readerControl$language, id),
+                        readerControl$reader(elem, readerControl$language, id),
                       pGetElem(x),
                       id = as.character(seq_along(x)),
                       SIMPLIFY = FALSE)
@@ -61,14 +104,13 @@ function(x, readerControl = list(reader = reader(x), language = "en"))
             counter <- counter + 1
         }
     }
+    x <- close(x)
 
-    structure(list(content = tdl,
-                   meta = CorpusMeta(),
-                   dmeta = data.frame(row.names = seq_along(tdl))),
-              class = c("VCorpus", "Corpus"))
+    as.VCorpus(tdl)
 }
 
 `[.PCorpus` <-
+`[.SimpleCorpus` <-
 function(x, i)
 {
     if (!missing(i)) {
@@ -77,7 +119,6 @@ function(x, i)
     }
     x
 }
-
 `[.VCorpus` <-
 function(x, i)
 {
@@ -106,12 +147,21 @@ function(x, i)
     db <- filehash::dbInit(x$dbcontrol[["dbName"]], x$dbcontrol[["dbType"]])
     filehash::dbFetch(db, x$content[[i]])
 }
+`[[.SimpleCorpus` <-
+function(x, i)
+{
+    i <- .map_name_index(x, i)
+    n <- names(x$content)
+    PlainTextDocument(x$content[[i]],
+                      id = if (is.null(n)) i else n[i],
+                      language = meta(x, "language"))
+}
 `[[.VCorpus` <-
 function(x, i)
 {
     i <- .map_name_index(x, i)
     if (!is.null(x$lazy))
-        .Call(copyCorpus, x, materialize(x, i))
+        .Call(tm_copyCorpus, x, materialize(x, i))
     x$content[[i]]
 }
 
@@ -136,12 +186,25 @@ function(x, i, value)
 
 as.list.PCorpus <- as.list.VCorpus <-
 function(x, ...)
-    content(x)
+    setNames(content(x), as.character(lapply(content(x), meta, "id")))
+
+as.list.SimpleCorpus <-
+function(x, ...)
+    as.list(content(x))
 
 as.VCorpus <-
 function(x)
     UseMethod("as.VCorpus")
 as.VCorpus.VCorpus <- identity
+as.VCorpus.list <-
+function(x)
+{
+    v <- list(content = x,
+              meta = CorpusMeta(),
+              dmeta = data.frame(row.names = seq_along(x)))
+    class(v) <- c("VCorpus", "Corpus")
+    v
+}
 
 outer_union <-
 function(x, y, ...)
@@ -168,21 +231,26 @@ function(..., recursive = FALSE)
     if (!all(unlist(lapply(args, inherits, class(x)))))
         stop("not all arguments are of the same corpus type")
 
-    structure(list(content = do.call("c", lapply(args, content)),
-                   meta = CorpusMeta(meta = do.call("c",
-                     lapply(args, function(a) meta(a, type = "corpus")))),
-                   dmeta = Reduce(outer_union, lapply(args, meta))),
-              class = c("VCorpus", "Corpus"))
+    v <- list(content = do.call("c", lapply(args, content)),
+              meta = CorpusMeta(meta = do.call("c",
+                lapply(args, function(a) meta(a, type = "corpus")))),
+              dmeta = Reduce(outer_union, lapply(args, meta)))
+    class(v) <- c("VCorpus", "Corpus")
+    v
 }
 
 content.VCorpus <-
 function(x)
 {
     if (!is.null(x$lazy))
-        .Call(copyCorpus, x, materialize(x))
+        .Call(tm_copyCorpus, x, materialize(x))
     x$content
 }
 
+content.SimpleCorpus <-
+function(x)
+    x$content
+
 content.PCorpus <-
 function(x)
 {
@@ -193,7 +261,9 @@ function(x)
 inspect <-
 function(x)
     UseMethod("inspect", x)
-inspect.PCorpus <- inspect.VCorpus <-
+inspect.PCorpus <-
+inspect.SimpleCorpus <-
+inspect.VCorpus <-
 function(x)
 {
     print(x)
@@ -202,11 +272,15 @@ function(x)
     invisible(x)
 }
 
-length.PCorpus <- length.VCorpus <-
+length.PCorpus <-
+length.SimpleCorpus <-
+length.VCorpus <-
 function(x)
     length(x$content)
 
-names.PCorpus <- names.VCorpus <-
+names.PCorpus <-
+names.SimpleCorpus <-
+names.VCorpus <-
 function(x)
     as.character(meta(x, "id", "local"))
 
@@ -217,15 +291,16 @@ function(x, value)
     x
 }
 
-print.PCorpus <- print.VCorpus <-
+format.PCorpus <-
+format.SimpleCorpus <-
+format.VCorpus <-
 function(x, ...)
 {
-    writeLines(sprintf("<<%s (documents: %d, metadata (corpus/indexed): %d/%d)>>",
-                       class(x)[1],
-                       length(x),
-                       length(meta(x, type = "corpus")),
-                       ncol(meta(x, type = "indexed"))))
-    invisible(x)
+    c(sprintf("<<%s>>", class(x)[1L]),
+      sprintf("Metadata:  corpus specific: %d, document level (indexed): %d",
+              length(meta(x, type = "corpus")),
+              ncol(meta(x, type = "indexed"))),
+      sprintf("Content:  documents: %d", length(x)))
 }
 
 writeCorpus <-
