@@ -22,6 +22,15 @@ function(x, ...)
     c(sprintf("<<%s>>", class(x)[1L]),
       sprintf("Metadata:  %d", length(meta(x))))
 
+inspect.TextDocument <-
+function(x)
+{
+    print(x)
+    cat("\n")
+    writeLines(as.character(x))
+    invisible(x)
+}
+
 PlainTextDocument <-
 function(x = character(0),
          author = character(0),
@@ -60,7 +69,8 @@ function(x, value)
 
 format.PlainTextDocument <-
 function(x, ...)
-    c(.format_TextDocument(x), sprintf("Content:  chars: %d", sum(nchar(x$content))))
+    c(.format_TextDocument(x),
+      sprintf("Content:  chars: %d", sum(nchar(x$content))))
 
 meta.PlainTextDocument <-
 function(x, tag = NULL, ...)
@@ -69,13 +79,14 @@ function(x, tag = NULL, ...)
 `meta<-.PlainTextDocument` <-
 function(x, tag = NULL, ..., value)
 {
-    if(is.null(tag))
+    if (is.null(tag))
         x$meta <- value
     else
         x$meta[[tag]] <- value
     x
 }
 
+words.character <-
 words.PlainTextDocument <-
 function(x, ...)
     scan_tokenizer(x)
