@@ -9,9 +9,12 @@ DocumentTermMatrix_classes <-
 function(x, weighting)
 {
     x <- as.simple_triplet_matrix(x)
-    if(!is.null(dimnames(x)))
+    if (!is.null(dimnames(x)))
         names(dimnames(x)) <- c("Terms", "Docs")
     class(x) <- TermDocumentMatrix_classes
+
+    if (is.null(weighting))
+        weighting <- weightTf
     ## <NOTE>
     ## Note that if weighting is a weight function, it already needs to
     ## know whether we have a term-document or document-term matrix.
@@ -27,63 +30,139 @@ function(x, weighting)
     ## in example("DocumentTermMatrix") fails [because weightTfIdf() is
     ## a weight function and not a weight function generator ...]
     ## Hence, for now, instead of
-    ##   if(inherits(weighting, "WeightFunction"))
+    ##   if (inherits(weighting, "WeightFunction"))
     ##      x <- weighting(x)
     ## use
-    if(is.function(weighting))
+    if (is.function(weighting))
         x <- weighting(x)
     ## and hope for the best ...
     ## </NOTE>
-    else if(is.character(weighting) && (length(weighting) == 2L))
+    else if (is.character(weighting) && (length(weighting) == 2L))
         attr(x, "weighting") <- weighting
-    else
-        stop("invalid weighting")
     x
 }
 
+.SimpleTripletMatrix <-
+function(i, j, v, terms, corpus)
+{
+    docs <- as.character(meta(corpus, "id", "local"))
+    if (length(docs) != length(corpus)) {
+        warning("invalid document identifiers")
+        docs <- NULL
+    }
+
+    simple_triplet_matrix(i, j, v,
+                          nrow = length(terms),
+                          ncol = length(corpus),
+                          dimnames = list(Terms = terms, Docs = docs))
+}
+
+filter_global_bounds <-
+function(m, bounds)
+{
+    m <- as.simple_triplet_matrix(m)
+
+    if (length(bounds) == 2L && is.numeric(bounds)) {
+        rs <- row_sums(m > 0)
+        m <- m[(rs >= bounds[1]) & (rs <= bounds[2]), ]
+    }
+    m
+}
+
 TermDocumentMatrix <-
 function(x, control = list())
     UseMethod("TermDocumentMatrix", x)
 
+TermDocumentMatrix.SimpleCorpus <-
+function(x, control = list())
+{
+    stopifnot(is.list(control))
+
+    txt <- content(x)
+
+    ## Conversion to lower case
+    if (is.null(control$tolower) || isTRUE(control$tolower))
+        txt <- tolower(txt)
+
+    ## Stopword filtering
+    .stopwords <- if (isTRUE(control$stopwords)) stopwords(meta(x, "language"))
+        else if (is.character(control$stopwords)) control$stopwords
+        else character(0)
+
+    .dictionary <- if (is.null(control$dictionary)) character(0)
+        else control$dictionary
+
+    ## Ensure local bounds
+    bl <- control$bounds$local
+    min_term_freq <-
+        if (length(bl) == 2L && is.numeric(bl) && bl[1] >= 0) bl[1] else 0L
+    max_term_freq <-
+        if (length(bl) == 2L && is.numeric(bl) && bl[2] >= 0)
+            min(bl[2], .Machine$integer.max) else .Machine$integer.max
+
+    ## Filter out too short or too long terms
+    wl <- control$wordLengths
+    min_word_length <- if (is.numeric(wl[1]) && wl[1] >= 0) wl[1] else 3L
+    max_word_length <- if (is.numeric(wl[2]) && wl[2] >= 0)
+        min(wl[2], .Machine$integer.max) else .Machine$integer.max
+
+    m <- tdm(txt,
+             isTRUE(control$removeNumbers),
+             .stopwords, .dictionary,
+             as.integer(min_term_freq), as.integer(max_term_freq),
+             as.integer(min_word_length), as.integer(max_word_length))
+
+    terms <- if (is.null(control$dictionary)) m$terms else control$dictionary
+    m <- .SimpleTripletMatrix(m$i, m$j, m$v, terms, x)
+
+    ## Stemming
+    ## <NOTE>
+    ## Ideally tdm() could perform stemming as well but there is no easy way to
+    ## access the SnowballC::wordStem() function from C++ (via Rcpp) without
+    ## significant overhead (as SnowballC does not export its internal C
+    ## functions).
+    ##
+    ## Stemming afterwards is still quite performant as we already have all
+    ## terms. However, there is some overhead involved as we need to recheck
+    ## local bounds and word lengths.
+    ## </NOTE>
+    if (isTRUE(control$stemming)) {
+        stems <- as.factor(SnowballC::wordStem(terms, meta(x, "language")))
+        m <- rollup(m, "Terms", stems)
+
+        ## Recheck local bounds
+        ## No need to check lower local bound as rollup aggregates frequencies
+        m[m > max_term_freq] <- 0
+
+        ## Recheck word lengths
+        terms_length <- nchar(rownames(m))
+        m <- m[min_word_length <= terms_length &
+               terms_length <= max_word_length, ]
+    }
+
+    m <- filter_global_bounds(m, control$bounds$global)
+
+    .TermDocumentMatrix(m, control$weighting)
+}
+
 TermDocumentMatrix.PCorpus <-
 TermDocumentMatrix.VCorpus <-
 function(x, control = list())
 {
     stopifnot(is.list(control))
 
-    tflist <- mclapply(unname(content(x)), termFreq, control)
-    tflist <- lapply(tflist, function(y) y[y > 0])
+    tflist <- tm_parLapply(unname(content(x)), termFreq, control)
 
     v <- unlist(tflist)
     i <- names(v)
-    allTerms <- sort(unique(as.character(if (is.null(control$dictionary)) i
-                                         else control$dictionary)))
-    i <- match(i, allTerms)
-    j <- rep(seq_along(x), sapply(tflist, length))
-    docs <- as.character(meta(x, "id", "local"))
-    if (length(docs) != length(x)) {
-        warning("invalid document identifiers")
-        docs <- NULL
-    }
+    terms <- sort(unique(as.character(if (is.null(control$dictionary)) i
+                                      else control$dictionary)))
+    i <- match(i, terms)
+    j <- rep.int(seq_along(x), lengths(tflist))
 
-    m <- simple_triplet_matrix(i = i, j = j, v = as.numeric(v),
-                               nrow = length(allTerms),
-                               ncol = length(x),
-                               dimnames =
-                                 list(Terms = allTerms,
-                                      Docs = docs))
-
-    bg <- control$bounds$global
-    if (length(bg) == 2L && is.numeric(bg)) {
-        rs <- row_sums(m > 0)
-        m <- m[(rs >= bg[1]) & (rs <= bg[2]), ]
-    }
-
-    weighting <- control$weighting
-    if (is.null(weighting))
-        weighting <- weightTf
-
-    .TermDocumentMatrix(m, weighting)
+    m <- .SimpleTripletMatrix(i, j, as.numeric(v), terms, x)
+    m <- filter_global_bounds(m, control$bounds$global)
+    .TermDocumentMatrix(m, control$weighting)
 }
 
 DocumentTermMatrix <-
@@ -104,7 +183,7 @@ as.TermDocumentMatrix.textcnt <-
 function(x, ...)
 {
     m <- simple_triplet_matrix(i = seq_along(x),
-                               j = rep(1, length(x)),
+                               j = rep_len(1L, length(x)),
                                v = as.numeric(x),
                                nrow = length(x),
                                ncol = 1,
@@ -144,7 +223,7 @@ function(x)
 {
     m <- NextMethod("t")
     attr(m, "weighting") <- attr(x, "weighting")
-    class(m) <- if(inherits(x, "DocumentTermMatrix"))
+    class(m) <- if (inherits(x, "DocumentTermMatrix"))
         TermDocumentMatrix_classes
     else
         DocumentTermMatrix_classes
@@ -154,7 +233,8 @@ function(x)
 termFreq <-
 function(doc, control = list())
 {
-    stopifnot(inherits(doc, "TextDocument"), is.list(control))
+    stopifnot(inherits(doc, "TextDocument") || is.character(doc),
+              is.list(control))
 
     ## Tokenize the corpus
     .tokenize <- control$tokenize
@@ -164,14 +244,14 @@ function(doc, control = list())
         .tokenize <- MC_tokenizer
     else if (identical(.tokenize, "scan"))
         .tokenize <- scan_tokenizer
-    else if (NLP::is.Span_Tokenizer(.tokenize))
-        .tokenize <- NLP::as.Token_Tokenizer(.tokenize)
+    else if (is.Span_Tokenizer(.tokenize))
+        .tokenize <- as.Token_Tokenizer(.tokenize)
     if (is.function(.tokenize))
         txt <- .tokenize(doc)
     else
         stop("invalid tokenizer")
 
-    ## Conversion to lower characters
+    ## Conversion to lower case
     .tolower <- control$tolower
     if (is.null(.tolower) || isTRUE(.tolower))
         .tolower <- tolower
@@ -192,17 +272,23 @@ function(doc, control = list())
     if (isTRUE(.removeNumbers))
         .removeNumbers <- removeNumbers
 
+    .language <- control$language
+    if (inherits(doc, "TextDocument"))
+        .language <- meta(doc, "language")
+    if (is.null(.language))
+        .language <- "en"
+
     ## Stopword filtering
     .stopwords <- control$stopwords
     if (isTRUE(.stopwords))
-        .stopwords <- function(x) x[is.na(match(x, stopwords(meta(doc, "language"))))] 
+        .stopwords <- function(x) x[is.na(match(x, stopwords(.language)))]
     else if (is.character(.stopwords))
         .stopwords <- function(x) x[is.na(match(x, control$stopwords))]
 
     ## Stemming
     .stemming <- control$stemming
     if (isTRUE(.stemming))
-        .stemming <- function(x) stemDocument(x, meta(doc, "language"))
+        .stemming <- function(x) stemDocument(x, .language)
 
     ## Default order for options which support reordering
     or <- c("removePunctuation", "removeNumbers", "stopwords", "stemming")
@@ -216,31 +302,22 @@ function(doc, control = list())
             txt <- g(txt)
     }
 
-    ## Check if the document content is NULL
-    if (is.null(txt))
-        return(setNames(integer(0), character(0)))
-
     ## If dictionary is set tabulate against it
     dictionary <- control$dictionary
-    tab <-  if (is.null(dictionary))
-        table(txt)
-    else
-        table(factor(txt, levels = dictionary))
+    tab <- table(if (is.null(dictionary)) txt else intersect(txt, dictionary))
 
     ## Ensure local bounds
     bl <- control$bounds$local
     if (length(bl) == 2L && is.numeric(bl))
-        tab <- tab[(tab >= bl[1]) & (tab <= bl[2])]
+        tab <- tab[(tab >= bl[1]) & (tab <= bl[2]), drop = FALSE]
 
     ## Filter out too short or too long terms
     nc <- nchar(names(tab), type = "chars")
     wl <- control$wordLengths
     lb <- if (is.numeric(wl[1])) wl[1] else 3
     ub <- if (is.numeric(wl[2])) wl[2] else Inf
-    tab <- tab[(nc >= lb) & (nc <= ub)]
+    tab <- tab[(nc >= lb) & (nc <= ub), drop = FALSE]
 
-    ## Return named integer
-    storage.mode(tab) <- "integer"
     class(tab) <- c("term_frequency", class(tab))
     tab
 }
@@ -255,9 +332,9 @@ function(x, ...)
     writeLines(sprintf("<<%s (%ss: %d, %ss: %d)>>",
                        class(x)[1], format[1L], nrow(x), format[2L], ncol(x)))
     writeLines(sprintf("Non-/sparse entries: %d/%.0f",
-                length(x$v), prod(dim(x)) - length(x$v)))
+                       length(x$v), prod(dim(x)) - length(x$v)))
     sparsity <- if (!prod(dim(x))) 100
-        else round((1 - length(x$v)/prod(dim(x))) * 100)
+        else round( (1 - length(x$v) / prod(dim(x))) * 100)
     writeLines(sprintf("Sparsity           : %s%%", sparsity))
     writeLines(sprintf("Maximal term length: %s",
                        max(nchar(Terms(x), type = "chars"), 0)))
@@ -271,8 +348,8 @@ inspect.DocumentTermMatrix <-
 function(x)
 {
     print(x)
-    cat("\n")
-    print(as.matrix(x))
+    cat("Sample             :\n")
+    print(as.matrix(sample.TermDocumentMatrix(x)))
 }
 
 `[.TermDocumentMatrix` <-
@@ -293,7 +370,7 @@ function(x, value)
 {
     x <- NextMethod("dimnames<-")
     dnx <- x$dimnames
-    if(!is.null(dnx))
+    if (!is.null(dnx))
         names(dnx) <- c("Docs", "Terms")
     x$dimnames <- dnx
     x
@@ -304,7 +381,7 @@ function(x, value)
 {
     x <- NextMethod("dimnames<-")
     dnx <- x$dimnames
-    if(!is.null(dnx))
+    if (!is.null(dnx))
         names(dnx) <- c("Terms", "Docs")
     x$dimnames <- dnx
     x
@@ -341,7 +418,7 @@ Terms.TermDocumentMatrix <-
 function(x)
 {
     s <- x$dimnames[[1L]]
-    if(is.null(s))
+    if (is.null(s))
         s <- rep.int(NA_character_, x$nrow)
     s
 }
@@ -351,7 +428,7 @@ Terms.DocumentTermMatrix <-
 function(x)
 {
     s <- x$dimnames[[2L]]
-    if(is.null(s))
+    if (is.null(s))
         s <- rep.int(NA_character_, x$ncol)
     s
 }
@@ -367,7 +444,7 @@ function(..., recursive = FALSE)
 {
     m <- lapply(list(...), as.TermDocumentMatrix)
 
-    if(length(m) == 1L)
+    if (length(m) == 1L)
         return(m[[1L]])
 
     weighting <- attr(m[[1L]], "weighting")
@@ -381,7 +458,7 @@ function(..., recursive = FALSE)
     j <- lapply(m, "[[", "j")
 
     m <- simple_triplet_matrix(i = match(allTermsNonUnique, allTerms),
-                               j = unlist(j) + rep.int(cs, sapply(j, length)),
+                               j = unlist(j) + rep.int(cs, lengths(j)),
                                v = unlist(lapply(m, "[[", "v")),
                                nrow = length(allTerms),
                                ncol = length(allDocs),
@@ -390,9 +467,9 @@ function(..., recursive = FALSE)
                                     Docs = allDocs))
     ## <NOTE>
     ## - We assume that all arguments have the same weighting
-    ## - Even if all matrices have the same input weighting it might be necessary
-    ##   to take additional steps (e.g., normalization for tf-idf or check for
-    ##   (0,1)-range for binary tf)
+    ## - Even if all matrices have the same input weighting it might be
+    ##   necessary to take additional steps (e.g., normalization for tf-idf or
+    ##   check for (0,1)-range for binary tf)
     ## </NOTE>
     .TermDocumentMatrix(m, weighting)
 }
@@ -410,7 +487,7 @@ function(x, lowfreq = 0, highfreq = Inf)
               is.numeric(lowfreq), is.numeric(highfreq))
 
     if (inherits(x, "DocumentTermMatrix")) x <- t(x)
-    rs <- slam::row_sums(x)
+    rs <- row_sums(x)
     names(rs[rs >= lowfreq & rs <= highfreq])
 }
 
@@ -428,7 +505,7 @@ function(x, terms, corlimit)
 
     j <- match(unique(terms), Terms(x), nomatch = 0L)
     suppressWarnings(
-        findAssocs(slam::crossapply_simple_triplet_matrix(x[, j], x[, -j], cor),
+        findAssocs(crossapply_simple_triplet_matrix(x[, j], x[, -j], cor),
                    terms, rep_len(corlimit, length(terms))))
 }
 findAssocs.matrix <-
@@ -444,7 +521,8 @@ function(x, terms, corlimit)
         if (!length(t))
             names(t) <- NULL
         t
-        }, i, corlimit)
+        },
+        i, corlimit)
 }
 
 removeSparseTerms <-
@@ -456,18 +534,32 @@ function(x, sparse)
     m <- if (inherits(x, "DocumentTermMatrix")) t(x) else x
     t <- table(m$i) > m$ncol * (1 - sparse)
     termIndex <- as.numeric(names(t[t]))
-    if (inherits(x, "DocumentTermMatrix")) x[, termIndex] else x[termIndex,]
+    if (inherits(x, "DocumentTermMatrix")) x[, termIndex] else x[termIndex, ]
+}
+
+sample.TermDocumentMatrix <-
+function(x, size = 10)
+{
+    stopifnot(inherits(x, c("DocumentTermMatrix", "TermDocumentMatrix")),
+              is.numeric(size), size >= 0)
+
+    m <- if (inherits(x, "DocumentTermMatrix")) t(x) else x
+    terms <- sort(names(sort(row_sums(m), decreasing = TRUE)
+                        [0:min(size, nTerms(m))]))
+    docs <- sort(names(sort(col_sums(m), decreasing = TRUE)
+                       [0:min(size, nDocs(m))]))
+    if (inherits(x, "DocumentTermMatrix")) x[docs, terms] else x[terms, docs]
 }
 
 CategorizedDocumentTermMatrix <-
 function(x, c)
 {
-    if(inherits(x, "TermDocumentMatrix"))
+    if (inherits(x, "TermDocumentMatrix"))
         x <- t(x)
-    else if(!inherits(x, "DocumentTermMatrix"))
+    else if (!inherits(x, "DocumentTermMatrix"))
         stop("wrong class")
 
-    if(length(c) != nDocs(x))
+    if (length(c) != nDocs(x))
         stop("invalid category ids")
 
     attr(x, "Category") <- c
@@ -477,3 +569,54 @@ function(x, c)
 
     x
 }
+
+findMostFreqTerms <-
+function(x, n = 6L, ...)
+    UseMethod("findMostFreqTerms")
+
+findMostFreqTerms.term_frequency <-
+function(x, n = 6L, ...)
+{
+    y <- x[order(x, decreasing = TRUE)[seq_len(n)]]
+    y[y > 0]
+}
+
+findMostFreqTerms.DocumentTermMatrix <-
+function(x, n = 6L, INDEX = NULL, ...)
+{
+    terms <- Terms(x)
+    if (!is.null(INDEX))
+        x <- rollup(x, 1L, INDEX)
+    f <- factor(x$i, seq_len(x$nrow))
+    js <- split(x$j, f)
+    vs <- split(x$v, f)
+    y <- Map(function(j, v, n) {
+                 p <- order(v, decreasing = TRUE)[seq_len(n)]
+                 v <- v[p]
+                 names(v) <- terms[j[p]]
+                 v
+             },
+             js, vs, pmin(lengths(vs), n))
+    names(y) <- x$dimnames[[1L]]
+    y
+}
+
+findMostFreqTerms.TermDocumentMatrix <-
+function(x, n = 6L, INDEX = NULL, ...)
+{
+    terms <- Terms(x)
+    if (!is.null(INDEX))
+        x <- rollup(x, 2L, INDEX)
+    f <- factor(x$j, seq_len(x$ncol))
+    is <- split(x$i, f)
+    vs <- split(x$v, f)
+    y <- Map(function(i, v, n) {
+                 p <- order(v, decreasing = TRUE)[seq_len(n)]
+                 v <- v[p]
+                 names(v) <- terms[i[p]]
+                 v
+             },
+             is, vs, pmin(lengths(vs), n))
+    names(y) <- x$dimnames[[2L]]
+    y
+}
